names = {'DISPATCHLOAD': 'PUBLIC_DVD_DISPATCHLOAD',
         'DUDETAILSUMMARY': 'PUBLIC_DVD_DUDETAILSUMMARY',
         'DUDETAIL': 'PUBLIC_DVD_DUDETAIL',
         'DISPATCHCONSTRAINT': 'PUBLIC_DVD_DISPATCHCONSTRAINT',
         'GENCONDATA': 'PUBLIC_DVD_GENCONDATA',
         'DISPATCH_UNIT_SCADA': 'PUBLIC_DVD_DISPATCH_UNIT_SCADA',
         'DISPATCHPRICE': 'PUBLIC_DVD_DISPATCHPRICE',
         'SPDREGIONCONSTRAINT': 'PUBLIC_DVD_SPDREGIONCONSTRAINT',
         'SPDCONNECTIONPOINTCONSTRAINT': 'PUBLIC_DVD_SPDCONNECTIONPOINTCONSTRAINT',
         'SPDINTERCONNECTORCONSTRAINT': 'PUBLIC_DVD_SPDINTERCONNECTORCONSTRAINT',
         'BIDPEROFFER_D': 'PUBLIC_DVD_BIDPEROFFER_D',
         'DISPATCHINTERCONNECTORRES': 'PUBLIC_DVD_DISPATCHINTERCONNECTORRES',
         'BIDDAYOFFER_D': 'PUBLIC_DVD_BIDDAYOFFER_D',
         'DISPATCHREGIONSUM': 'PUBLIC_DVD_DISPATCHREGIONSUM',
         'FCAS_4_SECOND': 'FCAS',
         'ELEMENTS_FCAS_4_SECOND': 'Elements_FCAS.csv',
         'VARIABLES_FCAS_4_SECOND': '820-0079 csv.csv',
         'Generators and Scheduled Loads': 'NEM Registration and Exemption List',
         'MNSP_INTERCONNECTOR': 'PUBLIC_DVD_MNSP_INTERCONNECTOR',
         'MNSP_PEROFFER': 'PUBLIC_DVD_MNSP_PEROFFER',
         'INTERCONNECTOR': 'PUBLIC_DVD_INTERCONNECTOR',
         'INTERCONNECTORCONSTRAINT': 'PUBLIC_DVD_INTERCONNECTORCONSTRAINT',
         'MNSP_DAYOFFER': 'PUBLIC_DVD_MNSP_DAYOFFER',
         'LOSSMODEL': 'PUBLIC_DVD_LOSSMODEL',
         'LOSSFACTORMODEL': 'PUBLIC_DVD_LOSSFACTORMODEL',
         'FCAS_4s_SCADA_MAP': '',
         'PLANTSTATS': '',
         'TRADINGLOAD': 'PUBLIC_DVD_TRADINGLOAD',
         'TRADINGPRICE': 'PUBLIC_DVD_TRADINGPRICE',
         'TRADINGREGIONSUM': 'PUBLIC_DVD_TRADINGREGIONSUM',
         'TRADINGINTERCONNECT': 'PUBLIC_DVD_TRADINGINTERCONNECT',
         'MARKET_PRICE_THRESHOLDS': 'PUBLIC_DVD_MARKET_PRICE_THRESHOLDS'}

table_types = {'DISPATCHLOAD': 'MMS',
               'DUDETAILSUMMARY': 'MMS',
               'DUDETAIL': 'MMS',
               'DISPATCHCONSTRAINT': 'MMS',
               'GENCONDATA': 'MMS',
               'DISPATCH_UNIT_SCADA': 'MMS',
               'DISPATCHPRICE': 'MMS',
               'SPDREGIONCONSTRAINT': 'MMS',
               'SPDCONNECTIONPOINTCONSTRAINT': 'MMS',
               'SPDINTERCONNECTORCONSTRAINT': 'MMS',
               'BIDPEROFFER_D': 'MMS',
               'DISPATCHINTERCONNECTORRES': 'MMS',
               'BIDDAYOFFER_D': 'MMS',
               'DISPATCHREGIONSUM': 'MMS',
               'FCAS_4_SECOND': 'FCAS',
               'ELEMENTS_FCAS_4_SECOND': 'STATIC',
               'VARIABLES_FCAS_4_SECOND': 'STATIC',
               'Generators and Scheduled Loads': 'STATICXL',
               'MNSP_INTERCONNECTOR': 'MMS',
               'MNSP_PEROFFER': 'MMS',
               'INTERCONNECTOR': 'MMS',
               'INTERCONNECTORCONSTRAINT': 'MMS',
               'MNSP_DAYOFFER': 'MMS',
               'LOSSMODEL': 'MMS',
               'LOSSFACTORMODEL': 'MMS',
               'FCAS_4s_SCADA_MAP': 'CUSTOM',
               'TRADINGLOAD': 'MMS',
               'TRADINGPRICE': 'MMS',
               'TRADINGREGIONSUM': 'MMS',
               'TRADINGINTERCONNECT': 'MMS',
               'MARKET_PRICE_THRESHOLDS': 'MMS'
               }

return_tables = list(names.keys())

display_as_AMEO = ['DISPATCHLOAD', 'DUDETAILSUMMARY', 'DUDETAIL', 'DISPATCHCONSTRAINT', 'GENCONDATA', 'DISPATCH_UNIT_SCADA',
                   'DISPATCHPRICE', 'SPDREGIONCONSTRAINT', 'SPDCONNECTIONPOINTCONSTRAINT', 'SPDINTERCONNECTORCONSTRAINT',
                   'BIDPEROFFER_D', 'DISPATCHINTERCONNECTORRES', 'BIDDAYOFFER_D', 'DISPATCHREGIONSUM', 'FCAS_4_SECOND',
                   'ELEMENTS_FCAS_4_SECOND', 'VARIABLES_FCAS_4_SECOND', 'Generators and Scheduled Loads', 'TRADINGLOAD',
                   'TRADINGPRICE', 'TRADINGREGIONSUM', 'TRADINGINTERCONNECT']

display_as_Custom = ['FCAS_4s_SCADA_MAP', 'PLANTSTATS']

static_tables = ['ELEMENTS_FCAS_4_SECOND', 'VARIABLES_FCAS_4_SECOND', 'Generators and Scheduled Loads']

static_table_url = {
    'ELEMENTS_FCAS_4_SECOND': 'https://www.aemo.com.au/-/media/Files/Electricity/NEM/Data/Ancillary_Services/Elements_FCAS.csv',
    'VARIABLES_FCAS_4_SECOND': 'https://www.aemo.com.au/-/media/Files/CSV/820-0079-csv.csv',
    'Generators and Scheduled Loads': 'https://www.aemo.com.au/-/media/Files/Electricity/NEM/Participant_Information/NEM-Registration-and-Exemption-List.xls'}

aemo_data_url = 'http://www.nemweb.com.au/Data_Archive/Wholesale_Electricity/MMSDM/{}/MMSDM_{}_{}/MMSDM_Historical_Data_SQLLoader/DATA/{}.zip'

fcas_4_url = 'http://www.nemweb.com.au/Reports/Current/Causer_Pays/FCAS_{}{}{}{}.zip'

fcas_4_url_hist = 'http://www.nemweb.com.au/Data_Archive/Wholesale_Electricity/FCAS_Causer_Pays/{}/FCAS_Causer_Pays_{}_{}/FCAS_{}{}{}{}.zip'

data_url = {'DISPATCHLOAD': 'aemo_data_url',
            'DUDETAILSUMMARY': 'aemo_data_url',
            'DUDETAIL': 'aemo_data_url',
            'DISPATCHCONSTRAINT': 'aemo_data_url',
            'GENCONDATA': 'aemo_data_url',
            'DISPATCH_UNIT_SCADA': 'aemo_data_url',
            'DISPATCHPRICE': 'aemo_data_url',
            'SPDREGIONCONSTRAINT': 'aemo_data_url',
            'SPDCONNECTIONPOINTCONSTRAINT': 'aemo_data_url',
            'SPDINTERCONNECTORCONSTRAINT': 'aemo_data_url',
            'BIDPEROFFER_D': 'aemo_data_url',
            'DISPATCHINTERCONNECTORRES': 'aemo_data_url',
            'INTERCONNECTOR': 'aemo_data_url',
            'INTERCONNECTORCONSTRAINT': 'aemo_data_url',
            'MNSP_INTERCONNECTOR': 'aemo_data_url',
            'BIDDAYOFFER_D': 'aemo_data_url',
            'DISPATCHREGIONSUM': 'aemo_data_url',
            'MNSP_DAYOFFER': 'aemo_data_url',
            'MNSP_PEROFFER': 'aemo_data_url',
            'LOSSMODEL': 'aemo_data_url',
            'LOSSFACTORMODEL': 'aemo_data_url',
            'DISPATCHCASESOLUTION': 'aemo_data_url',
            'FCAS': 'fcas_4_url',
            'TRADINGLOAD': 'aemo_data_url',
            'TRADINGPRICE': 'aemo_data_url',
            'TRADINGREGIONSUM': 'aemo_data_url',
            'TRADINGINTERCONNECT': 'aemo_data_url',
            'MARKET_PRICE_THRESHOLDS': 'aemo_data_url'
            }

filterable_cols = ['DUID', 'REGIONID', 'STATIONID', 'PARTICIPANTID', 'STARTTYPE', 'SCHEDULE_TYPE', 'GENCONID',
                   'BIDTYPE', 'VARIABLEID', 'INTERVENTION', 'DISPATCHMODE', 'STARTTYPE', 'CONNECTIONPOINTID',
                   'DISPATCHTYPE', 'CONSTRAINTID', 'PREDISPATCH', 'STPASA', 'MTPASA', 'LIMITTYPE', 'STATIONNAME',
                   'AGCFLAG', 'INTERCONNECTORID', 'NAME', 'Fuel Source - Primary', 'Fuel Source - Descriptor',
                   'Technology Type - Primary', 'Technology Type - Descriptor', 'ELEMENTNUMBER', 'MARKETNAME',
                   'VARIABLENUMBER', 'VARIABLETYPE', 'MMSDESCRIPTOR', 'ELEMENTTYPE', 'Region']

table_columns = {

    'DISPATCHLOAD': ['SETTLEMENTDATE', 'DUID', 'INTERVENTION', 'DISPATCHMODE', 'AGCSTATUS', 'INITIALMW',
                     'TOTALCLEARED', 'RAMPDOWNRATE', 'RAMPUPRATE', 'LOWER5MIN', 'LOWER60SEC',
                     'LOWER6SEC', 'RAISE5MIN', 'RAISE60SEC', 'RAISE6SEC', 'LOWERREG', 'RAISEREG',
                     'SEMIDISPATCHCAP', 'AVAILABILITY', 'RAISEREGENABLEMENTMAX', 'RAISEREGENABLEMENTMIN',
                     'LOWERREGENABLEMENTMAX', 'LOWERREGENABLEMENTMIN'],

    'TRADINGLOAD':['SETTLEMENTDATE', 'DUID', 'INITIALMW', 'TOTALCLEARED', 'RAMPDOWNRATE', 'RAMPUPRATE', 'LOWER5MIN',
                   'LOWER60SEC', 'LOWER6SEC', 'RAISE5MIN', 'RAISE60SEC', 'RAISE6SEC', 'LOWERREG', 'RAISEREG',
                   'SEMIDISPATCHCAP', 'AVAILABILITY'],

    'TRADINGPRICE': ['SETTLEMENTDATE', 'REGIONID', 'RRP', 'RAISE6SECRRP', 'RAISE60SECRRP',
                      'RAISE5MINRRP', 'RAISEREGRRP', 'LOWER6SECRRP', 'LOWER60SECRRP', 'LOWERREGRRP', 'PRICE_STATUS'],

    'TRADINGREGIONSUM': ['SETTLEMENTDATE', 'REGIONID', 'TOTALDEMAND',
                          'AVAILABLEGENERATION', 'AVAILABLELOAD', 'DEMANDFORECAST', 'DISPATCHABLEGENERATION',
                          'DISPATCHABLELOAD', 'NETINTERCHANGE', 'EXCESSGENERATION', 'LOWER5MINLOCALDISPATCH',
                          'LOWER60SECLOCALDISPATCH', 'LOWER6SECLOCALDISPATCH', 'RAISE5MINLOCALDISPATCH',
                          'RAISE60SECLOCALDISPATCH', 'RAISE6SECLOCALDISPATCH', 'LOWERREGLOCALDISPATCH',
                          'RAISEREGLOCALDISPATCH', 'INITIALSUPPLY', 'CLEAREDSUPPLY', 'TOTALINTERMITTENTGENERATION',
                          'DEMAND_AND_NONSCHEDGEN', 'UIGF', 'SEMISCHEDULE_CLEAREDMW', 'SEMISCHEDULE_COMPLIANCEMW'],

    'TRADINGINTERCONNECT': ['SETTLEMENTDATE', 'INTERCONNECTORID', 'MWFLOW', 'METEREDMWFLOW', 'MWLOSSES'],

    'DUDETAILSUMMARY': ['DUID', 'START_DATE', 'END_DATE', 'DISPATCHTYPE', 'CONNECTIONPOINTID', 'REGIONID', 'STATIONID',
                        'PARTICIPANTID', 'LASTCHANGED', 'TRANSMISSIONLOSSFACTOR', 'STARTTYPE', 'DISTRIBUTIONLOSSFACTOR',
                        'SCHEDULE_TYPE', 'MAX_RAMP_RATE_UP', 'MAX_RAMP_RATE_DOWN'],

    'DISPATCHCONSTRAINT': ['SETTLEMENTDATE', 'RUNNO', 'CONSTRAINTID', 'INTERVENTION', 'RHS', 'MARGINALVALUE',
                           'VIOLATIONDEGREE', 'LASTCHANGED', 'GENCONID_EFFECTIVEDATE', 'GENCONID_VERSIONNO', 'LHS',
                           'DISPATCHINTERVAL'],

    'GENCONDATA': ['GENCONID', 'EFFECTIVEDATE', 'VERSIONNO', 'CONSTRAINTTYPE', 'CONSTRAINTVALUE', 'DESCRIPTION',
                   'GENERICCONSTRAINTWEIGHT', 'LASTCHANGED', 'DISPATCH', 'PREDISPATCH', 'STPASA', 'MTPASA',
                   'LIMITTYPE', 'REASON'],

    'DISPATCH_UNIT_SCADA': ['SETTLEMENTDATE', 'DUID', 'SCADAVALUE'],

    'DUDETAIL': ['EFFECTIVEDATE', 'DUID', 'VERSIONNO', 'CONNECTIONPOINTID', 'REGISTEREDCAPACITY', 'AGCCAPABILITY',
                 'DISPATCHTYPE', 'MAXCAPACITY', 'STARTTYPE', 'NORMALLYONFLAG', 'LASTCHANGED'],

    'DISPATCHPRICE': ['SETTLEMENTDATE', 'REGIONID', 'INTERVENTION', 'RRP', 'RAISE6SECRRP', 'RAISE60SECRRP',
                      'RAISE5MINRRP', 'RAISEREGRRP', 'LOWER6SECRRP', 'LOWER60SECRRP', 'LOWER5MINRRP', 'LOWERREGRRP', 
                      'PRICE_STATUS'],

    'SPDREGIONCONSTRAINT': ['REGIONID', 'EFFECTIVEDATE', 'VERSIONNO', 'GENCONID', 'FACTOR', 'LASTCHANGED', 'BIDTYPE'],

    'SPDCONNECTIONPOINTCONSTRAINT': ['CONNECTIONPOINTID', 'EFFECTIVEDATE', 'VERSIONNO', 'GENCONID', 'FACTOR', 'BIDTYPE',
                                     'LASTCHANGED'],

    'SPDINTERCONNECTORCONSTRAINT': ['INTERCONNECTORID', 'EFFECTIVEDATE', 'VERSIONNO', 'GENCONID', 'FACTOR',
                                    'LASTCHANGED'],

    'BIDPEROFFER_D': ['DUID', 'BANDAVAIL1', 'BANDAVAIL2', 'BANDAVAIL3', 'BANDAVAIL4', 'BANDAVAIL5', 'BANDAVAIL6',
                      'BANDAVAIL7', 'BANDAVAIL8', 'BANDAVAIL9', 'BANDAVAIL10', 'MAXAVAIL', 'RAMPUPRATE',
                      'RAMPDOWNRATE', 'BIDTYPE', 'SETTLEMENTDATE', 'ENABLEMENTMIN', 'ENABLEMENTMAX', 'LOWBREAKPOINT',
                      'HIGHBREAKPOINT', 'INTERVAL_DATETIME', 'OFFERDATE'],

    'DISPATCHINTERCONNECTORRES': ['SETTLEMENTDATE', 'INTERCONNECTORID', 'DISPATCHINTERVAL', 'INTERVENTION', 'MWFLOW',
                                  'METEREDMWFLOW', 'MWLOSSES'],

    'INTERCONNECTOR': ['INTERCONNECTORID', 'REGIONFROM', 'REGIONTO', 'LASTCHANGED'],

    'INTERCONNECTORCONSTRAINT': ['INTERCONNECTORID', 'FROMREGIONLOSSSHARE', 'EFFECTIVEDATE', 'VERSIONNO',
                                 'LOSSCONSTANT', 'LOSSFLOWCOEFFICIENT', 'ICTYPE'],

    'MNSP_INTERCONNECTOR': ['INTERCONNECTORID', 'LINKID', 'FROMREGION', 'TOREGION', 'MAXCAPACITY', 'FROM_REGION_TLF',
                            'TO_REGION_TLF', 'LHSFACTOR', 'EFFECTIVEDATE', 'VERSIONNO'],

    'BIDDAYOFFER_D': ['SETTLEMENTDATE', 'DUID', 'BIDTYPE', 'OFFERDATE', 'VERSIONNO', 'PRICEBAND1', 'PRICEBAND2',
                      'PRICEBAND3', 'PRICEBAND4', 'PRICEBAND5', 'PRICEBAND6', 'PRICEBAND7', 'PRICEBAND8',
                      'PRICEBAND9', 'PRICEBAND10', 'T1', 'T2', 'T3', 'T4', 'MINIMUMLOAD'],


    'DISPATCHREGIONSUM': ['SETTLEMENTDATE', 'REGIONID', 'DISPATCHINTERVAL', 'INTERVENTION', 'TOTALDEMAND',
                          'AVAILABLEGENERATION', 'AVAILABLELOAD', 'DEMANDFORECAST', 'DISPATCHABLEGENERATION',
                          'DISPATCHABLELOAD', 'NETINTERCHANGE', 'EXCESSGENERATION', 'LOWER5MINLOCALDISPATCH',
                          'LOWER60SECLOCALDISPATCH', 'LOWER6SECLOCALDISPATCH', 'RAISE5MINLOCALDISPATCH',
                          'RAISE60SECLOCALDISPATCH', 'RAISE6SECLOCALDISPATCH', 'LOWERREGLOCALDISPATCH',
                          'RAISEREGLOCALDISPATCH', 'INITIALSUPPLY', 'CLEAREDSUPPLY', 'TOTALINTERMITTENTGENERATION',
                          'DEMAND_AND_NONSCHEDGEN', 'UIGF', 'SEMISCHEDULE_CLEAREDMW', 'SEMISCHEDULE_COMPLIANCEMW'],

    'MNSP_PEROFFER': ['SETTLEMENTDATE', 'OFFERDATE', 'VERSIONNO', 'PARTICIPANTID', 'LINKID', 'PERIODID',
                      'BANDAVAIL1', 'BANDAVAIL2', 'BANDAVAIL3', 'BANDAVAIL4', 'BANDAVAIL5', 'BANDAVAIL6',
                      'BANDAVAIL7', 'BANDAVAIL8', 'BANDAVAIL9', 'BANDAVAIL10'],

    'MNSP_DAYOFFER': ['SETTLEMENTDATE', 'OFFERDATE', 'VERSIONNO', 'PARTICIPANTID', 'LINKID', 'PERIODID',
                      'PRICEBAND1', 'PRICEBAND2', 'PRICEBAND3', 'PRICEBAND4', 'PRICEBAND5', 'PRICEBAND6',
                      'PRICEBAND7', 'PRICEBAND8', 'PRICEBAND9', 'PRICEBAND10'],

    'LOSSMODEL': ['EFFECTIVEDATE', 'VERSIONNO', 'INTERCONNECTORID', 'LOSSSEGMENT', 'MWBREAKPOINT'],

    'LOSSFACTORMODEL': ['EFFECTIVEDATE', 'VERSIONNO', 'INTERCONNECTORID', 'REGIONID', 'DEMANDCOEFFICIENT'],

    'DISPATCHCASESOLUTION': ['SETTLEMENTDATE', 'TOTALOBJECTIVE'],

    'FCAS_4_SECOND': ['TIMESTAMP', 'ELEMENTNUMBER', 'VARIABLENUMBER', 'VALUE', 'VALUEQUALITY'],

    'ELEMENTS_FCAS_4_SECOND': ['ELEMENTNUMBER', 'EMSNAME', 'ELEMENTTYPE', 'MMSDESCRIPTOR'],

    'VARIABLES_FCAS_4_SECOND': ['VARIABLENUMBER', 'VARIABLETYPE'],

    'Generators and Scheduled Loads': ['Participant', 'Station Name', 'Region', 'Dispatch Type', 'Category', 'Classification',
                                 'Fuel Source - Primary', 'Fuel Source - Descriptor', 'Technology Type - Primary',
                                 'Technology Type - Descriptor', 'Aggregation', 'DUID'],

    'FCAS_4s_SCADA_MAP': ['ELEMENTNUMBER', 'MARKETNAME', 'ERROR'],
    'PLANTSTATS': ['Month', 'DUID', 'CapacityFactor', 'Volume', 'TRADING_VWAP', 'DISPATCH_VWAP',
                   'NodalPeakCapacityFactor', 'Nodal90thPercentileCapacityFactor'],
    'MARKET_PRICE_THRESHOLDS': ['EFFECTIVEDATE', 'VERSIONNO', 'VOLL', 'MARKETPRICEFLOOR']}

table_primary_keys = {'DISPATCHCONSTRAINT': ['CONSTRAINTID', 'GENCONID_EFFECTIVEDATE', 'GENCONID_VERSIONNO',
                                             'SETTLEMENTDATE', 'INTERVENTION'],
                      'DUDETAILSUMMARY': ['DUID', 'START_DATE', 'END_DATE'], 'STATION': ['STATIONID'],
                      'DUDETAIL': ['EFFECTIVEDATE', 'DUID', 'VERSIONNO'],
                      'SPDREGIONCONSTRAINT': ['EFFECTIVEDATE', 'GENCONID', 'REGIONID', 'VERSIONNO', 'BIDTYPE'],
                      'SPDCONNECTIONPOINTCONSTRAINT': ['EFFECTIVEDATE', 'GENCONID', 'CONNECTIONPOINTID', 'VERSIONNO',
                                                       'BIDTYPE'],
                      'SPDINTERCONNECTORCONSTRAINT': ['EFFECTIVEDATE', 'GENCONID', 'INTERCONNECTORID', 'VERSIONNO'],
                      'GENCONDATA': ['GENCONID', 'EFFECTIVEDATE', 'VERSIONNO'],
                      'MNSP_PEROFFER': ['SETTLEMENTDATE', 'OFFERDATE', 'VERSIONNO', 'PARTICIPANTID', 'LINKID', 'PERIODID'],
                      'MNSP_DAYOFFER': ['SETTLEMENTDATE', 'OFFERDATE', 'VERSIONNO', 'PARTICIPANTID', 'LINKID'],
                      'INTERCONNECTORCONSTRAINT': ['EFFECTIVEDATE', 'INTERCONNECTORID', 'VERSIONNO'],
                      'MNSP_INTERCONNECTOR': ['EFFECTIVEDATE', 'LINKID', 'VERSIONNO'],
                      'LOSSMODEL': ['EFFECTIVEDATE', 'INTERCONNECTORID', 'LOSSSEGMENT', 'VERSIONNO'],
                      'LOSSFACTORMODEL': ['EFFECTIVEDATE', 'INTERCONNECTORID', 'REGIONID', 'VERSIONNO'],
                      'BIDPEROFFER_D': ['BIDTYPE', 'DUID', 'OFFERDATE', 'INTERVAL_DATETIME', 'SETTLEMENTDATE'],
                      'DISPATCHINTERCONNECTORRES': ['DISPATCHINTERVAL', 'INTERCONNECTORID', 'INTERVENTION',
                                                    'SETTLEMENTDATE'],
                      'INTERCONNECTOR': ['INTERCONNECTORID'],
                      'DISPATCHPRICE': ['INTERVENTION', 'REGIONID', 'SETTLEMENTDATE'],
                      'BIDDAYOFFER_D': ['BIDTYPE', 'DUID', 'SETTLEMENTDATE'],
                      'DISPATCHREGIONSUM': ['DISPATCHINTERVAL', 'INTERVENTION', 'REGIONID', 'SETTLEMENTDATE'],
                      'DISPATCHLOAD': ['SETTLEMENTDATE', 'INTERVENTION', 'DUID'],
                      'DISPATCH_UNIT_SCADA': ['SETTLEMENTDATE', 'DUID'],
                      'FCAS_4_SECOND': ['TIMESTAMP', 'ELEMENTNUMBER', 'VARIABLENUMBER'],
                      'ELEMENTS_FCAS_4_SECOND': ['ELEMENTNUMBER'],
                      'VARIABLES_FCAS_4_SECOND': ['VARIABLENUMBER', 'VARIABLETYPE'],
                      'Generators and Scheduled Loads': ['DUID'],
                      'FCAS_4s_SCADA_MAP': ['ELEMENTNUMBER', 'MARKETNAME'],
                      'TRADINGLOAD': ['SETTLEMENTDATE', 'DUID'],
                      'TRADINGPRICE': ['SETTLEMENTDATE', 'REGIONID'],
                      'TRADINGREGIONSUM': ['SETTLEMENTDATE', 'REGIONID'],
                      'TRADINGINTERCONNECT': ['SETTLEMENTDATE', 'INTERCONNECTORID'],
                      'PLANTSTATS': ['Month', 'DUID'],
                      'MARKET_PRICE_THRESHOLDS': ['EFFECTIVEDATE', 'VERSIONNO']}

effective_date_group_col = {'SPDREGIONCONSTRAINT': ['GENCONID'],
                            'SPDCONNECTIONPOINTCONSTRAINT': ['GENCONID'],
                            'SPDINTERCONNECTORCONSTRAINT': ['GENCONID'],
                            'GENCONDATA': ['GENCONID'],
                            'MNSP_INTERCONNECTOR': ['INTERCONNECTORID'],
                            'INTERCONNECTORCONSTRAINT': ['INTERCONNECTORID'],
                            'INTERCONNECTOR': ['INTERCONNECTORID'],
                            'LOSSMODEL': ['INTERCONNECTORID'],
                            'LOSSFACTORMODEL': ['INTERCONNECTORID'],
                            'DUDETAILSUMMARY': ['DUID'],
                            'MNSP_PEROFFER': ['LINKID'],
                            'MNSP_DAYOFFER': ['LINKID'],
                            'DUDETAIL': ['DUID'],
                            'MARKET_PRICE_THRESHOLDS': []}

primary_date_columns = {'DISPATCHLOAD': 'SETTLEMENTDATE',
                        'TRADINGLOAD': 'SETTLEMENTDATE',
                        'TRADINGPRICE': 'SETTLEMENTDATE',
                        'TRADINGREGIONSUM': 'SETTLEMENTDATE',
                        'TRADINGINTERCONNECT': 'SETTLEMENTDATE',
                        'DUDETAILSUMMARY': 'START_DATE',
                        'DUDETAIL': 'EFFECTIVEDATE',
                        'DISPATCHCONSTRAINT': 'SETTLEMENTDATE',
                        'GENCONDATA': 'EFFECTIVEDATE',
                        'DISPATCH_UNIT_SCADA': 'SETTLEMENTDATE',
                        'DISPATCHPRICE': 'SETTLEMENTDATE',
                        'SPDREGIONCONSTRAINT': 'EFFECTIVEDATE',
                        'SPDCONNECTIONPOINTCONSTRAINT': 'EFFECTIVEDATE',
                        'SPDINTERCONNECTORCONSTRAINT': 'EFFECTIVEDATE',
                        'BIDPEROFFER_D': 'INTERVAL_DATETIME',
                        'DISPATCHINTERCONNECTORRES': 'SETTLEMENTDATE',
                        'BIDDAYOFFER_D': 'SETTLEMENTDATE',
                        'DISPATCHREGIONSUM': 'SETTLEMENTDATE',
                        'FCAS_4_SECOND': 'TIMESTAMP',
                        'ELEMENTS_FCAS_4_SECOND': None,
                        'VARIABLES_FCAS_4_SECOND': None,
                        'Generators and Scheduled Loads': None,
                        'MNSP_INTERCONNECTOR': 'EFFECTIVEDATE',
                        'MNSP_PEROFFER': 'SETTLEMENTDATE',
                        'INTERCONNECTOR': 'LASTCHANGED',
                        'INTERCONNECTORCONSTRAINT': 'EFFECTIVEDATE',
                        'MNSP_DAYOFFER': 'SETTLEMENTDATE',
                        'LOSSMODEL': 'EFFECTIVEDATE',
                        'LOSSFACTORMODEL': 'EFFECTIVEDATE',
                        'FCAS_4s_SCADA_MAP': None,
                        'MARKET_PRICE_THRESHOLDS': 'EFFECTIVEDATE'}

months = ['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12']

nem_data_model_start_time = '2009/07/01 00:00:00'


# GUI settings.
header_y_pad = 30
query_y_pad = (20, 0)
query_row_offset = 2
row_height = 6
names_internal_row = 1
table_list_internal_row = 1
start_time_label_internal_row = 2
start_time_internal_row = 3
end_time_label_internal_row = 4
end_time_internal_row = 5
plus_internal_row = 6
plus_merge_internal_row = 7
list_row_span = 5
list_column_span = 1
save_field_column_span = 3
standard_x_pad = (0, 10)
list_filter_row_span = 4
internal_filter_row = 2
delete_button_internal_row = 5
last_column = 100
join_type = ['inner', 'left', 'right']

# Testting settings
raw_data_cache = 'E:/raw_aemo_data'



import unittest
from nemosis import data_fetch_methods
import pandas as pd
from nemosis import custom_tables
import math
import numpy as np
import time
from nemosis import defaults
from datetime import datetime, timedelta
import os


class TestBaseVolumeWeightAveragePriceFunction(unittest.TestCase):
    def setUp(self):
        self.volume = pd.Series([55, 0, math.nan, 60, 40])
        self.price = pd.Series([88, 90, -100, -100, 50])
        self.pricing_data = pd.DataFrame({
             'SCADAVALUE': [8, 9, 50, 11, 10, 0],
             'TRADING_TOTALCLEARED': [20, math.nan, math.nan, 80, math.nan, math.nan],
             'TRADING_RRP': [80, math.nan, math.nan, 100, math.nan, math.nan],
             'DISPATCH_RRP': [80, 90, 89, 111, 110, 75]})
        pass

    def test_volume_weighted_average_price(self):
        vwap = custom_tables.volume_weighted_average_price(self.volume, self.price)
        self.assertAlmostEqual(vwap, 5.419354839, 4)

    def test_trading_price(self):
        vwap = custom_tables.volume_weighted_average_trading_price(self.pricing_data)
        self.assertAlmostEqual(vwap, 96, 4)

    def test_spot_price(self):
        vwap = custom_tables.volume_weighted_average_spot_price(self.pricing_data)
        self.assertAlmostEqual(vwap, 93.42045455, 4)


class TestPerformanceAtNodalPeak(unittest.TestCase):
    def setUp(self):
        self.cap_and_output = pd.DataFrame(
            {'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00', '2015/01/01 00:10:00',
                                '2015/01/01 12:00:00', '2015/01/01 12:10:00', '2015/01/01 22:10:00'],
             'DUID': ['A', 'A', 'A', 'A', 'A', 'A'],
             'MAXCAPACITY': [68, 68, 68, 68, 68, 68],
             'SCADAVALUE': [8, 9, 50, 11, 10, 0],
             'TOTALDEMAND': [1000, 1100, 13000, 900, 800, 1000]})
        self.cap_and_output['SETTLEMENTDATE'] = pd.to_datetime(self.cap_and_output['SETTLEMENTDATE'])
        self.cap_and_output_nans = pd.DataFrame(
            {'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00', '2015/01/01 00:10:00',
                                '2015/01/01 12:00:00', '2015/01/01 12:10:00', '2015/01/01 22:10:00'],
             'DUID': ['A', 'A', 'A', 'A', 'A', 'A'],
             'MAXCAPACITY': [68, 68, 68, 68, 68, 68],
             'SCADAVALUE': [8, 9, math.nan, 11, 10, 0],
             'TOTALDEMAND': [1000, 1100, 13000, 900, 800, 1000]})
        self.cap_and_output_nans['SETTLEMENTDATE'] = pd.to_datetime(self.cap_and_output['SETTLEMENTDATE'])

        pass

    def test_performance_at_nodal_peak(self):
        peak = custom_tables.performance_at_nodal_peak(self.cap_and_output)
        self.assertAlmostEqual(peak, 50/68, 4)

    def test_performance_at_nodal_peak_nans(self):
        peak = custom_tables.performance_at_nodal_peak(self.cap_and_output_nans)
        self.assertAlmostEqual(peak, 0.0, 4)


class TestCapacityFactor90(unittest.TestCase):
    def setUp(self):
        self.cap_and_output = pd.DataFrame(
            {'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00', '2015/01/01 00:10:00',
                                '2015/01/01 12:00:00', '2015/01/01 12:10:00', '2015/01/01 22:10:00'],
             'DUID': ['A', 'A', 'A', 'A', 'A', 'A'],
             'MAXCAPACITY': [68, 68, 68, 68, 68, 68],
             'SCADAVALUE': [8, 9, 50, 11, 10, 0],
             'TOTALDEMAND': [1000, 1100, 13000, 900, 800, 1000]})
        self.cap_and_output['SETTLEMENTDATE'] = pd.to_datetime(self.cap_and_output['SETTLEMENTDATE'])
        self.cap_and_output2 = pd.DataFrame(
            {'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00', '2015/01/01 00:10:00',
                                '2015/01/01 12:00:00', '2015/01/01 12:10:00', '2015/01/01 22:10:00',
                                '2015/01/02 00:00:00', '2015/01/02 00:05:00', '2015/01/02 00:10:00',
                                '2015/01/02 12:00:00', '2015/01/02 12:10:00', '2015/01/02 22:10:00'],
             'DUID': ['A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A'],
             'MAXCAPACITY': [68, 68, 68, 68, 68, 68, 68, 68, 68, 68, 68, 68],
             'SCADAVALUE': [8, 9, 50, 11, 10, 0, 8, 9, 50, 11, 10, 90],
             'TOTALDEMAND': [10000, 11000, 13000, 900, 800, 1000, 1000, 11000, 13000, 900, 800, 18000]})
        self.cap_and_output2['SETTLEMENTDATE'] = pd.to_datetime(self.cap_and_output2['SETTLEMENTDATE'])
        self.cap_and_output2_nans = pd.DataFrame(
            {'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00', '2015/01/01 00:10:00',
                                '2015/01/01 12:00:00', '2015/01/01 12:10:00', '2015/01/01 22:10:00',
                                '2015/01/02 00:00:00', '2015/01/02 00:05:00', '2015/01/02 00:10:00',
                                '2015/01/02 12:00:00', '2015/01/02 12:10:00', '2015/01/02 22:10:00'],
             'DUID': ['A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A'],
             'MAXCAPACITY': [68, 68, 68, 68, 68, 68, 68, 68, 68, 68, 68, 68],
             'SCADAVALUE': [8, 9, 50, 11, 10, 0, 8, 9, 50, 11, 10, math.nan],
             'TOTALDEMAND': [10000, 11000, 13000, 900, 800, 1000, 1000, 11000, 13000, 900, 800, 18000]})
        self.cap_and_output2_nans['SETTLEMENTDATE'] = pd.to_datetime(self.cap_and_output2['SETTLEMENTDATE'])
        self.cap_and_output2_nans2 = pd.DataFrame(
            {'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00', '2015/01/01 00:10:00',
                                '2015/01/01 12:00:00', '2015/01/01 12:10:00', '2015/01/01 22:10:00',
                                '2015/01/02 00:00:00', '2015/01/02 00:05:00', '2015/01/02 00:10:00',
                                '2015/01/02 12:00:00', '2015/01/02 12:10:00', '2015/01/02 22:10:00'],
             'DUID': ['A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'A'],
             'MAXCAPACITY': [math.nan, math.nan, math.nan, 68, 68, 68, 68, 68, 68, 68, 68, 68],
             'SCADAVALUE': [math.nan, math.nan, math.nan, 11, 10, 0, 8, 9, 50, 11, 10, 90],
             'TOTALDEMAND': [10000, 11000, 13000, 900, 800, 1000, 1000, 11000, 13000, 900, 800, 18000]})
        self.cap_and_output2_nans2['SETTLEMENTDATE'] = pd.to_datetime(self.cap_and_output2['SETTLEMENTDATE'])
        pass

    def test_capacity_factor(self):
        peak = custom_tables.capacity_factor_over_90th_percentile_of_nodal_demand(self.cap_and_output)
        self.assertAlmostEqual(peak, 50/68, 4)

    def test_capacity_factor_2_intervals(self):
        peak = custom_tables.capacity_factor_over_90th_percentile_of_nodal_demand(self.cap_and_output2)
        self.assertAlmostEqual(peak, (50/68 + 90/68) / 2, 4)

    def test_capacity_factor_2_intervals_one_nan(self):
        peak = custom_tables.capacity_factor_over_90th_percentile_of_nodal_demand(self.cap_and_output2_nans)
        self.assertAlmostEqual(peak, (50/68 + 0.0/68) / 2, 4)

    def test_capacity_factor_2_intervals_plant_built_after_first(self):
        peak = custom_tables.capacity_factor_over_90th_percentile_of_nodal_demand(self.cap_and_output2_nans2)
        self.assertAlmostEqual(peak, (90 / 68), 4)


class TestCapacityScadaBasedStats(unittest.TestCase):
    def setUp(self):
        self.cap_and_output = pd.DataFrame({'MAXCAPACITY': [68, 68, 68, 68, 68], 'SCADAVALUE': [8, 9, 50, 11, 10]})
        pass

    def test_capacity_factor(self):
        cf = custom_tables.capacity_factor(self.cap_and_output)
        self.assertAlmostEqual(cf, 0.258823529, 4)

    def test_volume(self):
        v = custom_tables.volume(self.cap_and_output)
        self.assertAlmostEqual(v, 7.3333, 4)


class TestMonthlyGroupingForStats(unittest.TestCase):
    def setUp(self):
        self.cap_and_output = pd.DataFrame(
            {'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00', '2015/01/01 00:10:00',
                                '2015/02/01 00:00:00', '2015/02/01 00:05:00', '2015/02/01 00:10:00'],
             'DUID': ['A', 'A', 'A', 'A', 'A', 'A'],
             'MAXCAPACITY': [68, 68, 68, 68, 68, 68],
             'SCADAVALUE': [8, 9, 50, 11, 10, 0],
             'TRADING_TOTALCLEARED': [111, math.nan, math.nan, 250, math.nan, math.nan],
             'TRADING_RRP': [115, math.nan, math.nan, 250, math.nan, math.nan],
             'DISPATCH_RRP': [112, 97, 102, 81, 85, 91],
             'TOTALDEMAND': [1000, 1020, 1100, 990, 1100, 897]})
        self.cap_and_output['SETTLEMENTDATE'] = pd.to_datetime(self.cap_and_output['SETTLEMENTDATE'])
        self.cf_by_month = pd.DataFrame(
            {'Month': ['2015-01', '2015-02'],
             'DUID': ['A', 'A'],
             'CapacityFactor': [0.328431373, 0.102941176],
             'Volume': [67.0/12, 21.0/12],
             'TRADING_VWAP': [115.0, 250.0],
             'DISPATCH_VWAP': [102.5223881, 82.9047619],
             'NodalPeakCapacityFactor': [0.735294118, 0.147058824],
             'Nodal90thPercentileCapacityFactor': [0.735294118, 0.147058824]})
        pass

    def test_one_duid_two_months_example(self):
        cf_by_month_and_duid = custom_tables.stats_by_month_and_plant(self.cap_and_output)
        cf_by_month_and_duid = cf_by_month_and_duid.reset_index(drop=True)
        pd.testing.assert_frame_equal(cf_by_month_and_duid, self.cf_by_month)


class TestMergeTables(unittest.TestCase):
    def setUp(self):
        self.gen_info = pd.DataFrame({
            'EFFECTIVEDATE': ['2014/01/01 00:00:00', '2017/01/01 00:00:00', '2014/01/01 00:00:00'],
            'DUID': ['A', 'A', 'B'],
            'MAXCAPACITY': [333, 400, 250]})
        self.gen_info['EFFECTIVEDATE'] = pd.to_datetime(self.gen_info['EFFECTIVEDATE'])
        self.gen_info2 = pd.DataFrame({
            'START_DATE': ['2014/01/01 00:00:00', '2017/01/01 00:00:00', '2014/01/01 00:00:00'],
            'DUID': ['A', 'A', 'B'],
            'REGIONID': ['NSW1', 'NSW1', 'VIC1']})
        self.gen_info2['START_DATE'] = pd.to_datetime(self.gen_info2['START_DATE'])
        self.scada = pd.DataFrame({
            'DUID': ['A', 'A', 'B', 'B'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00',
                               '2015/01/01 00:00:00', '2015/01/01 00:05:00'],
            'SCADAVALUE': [150, 150,  200, 220]})
        self.scada['SETTLEMENTDATE'] = pd.to_datetime(self.scada['SETTLEMENTDATE'])
        self.trading_load = pd.DataFrame({
            'DUID': ['A', 'B'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:00:00'],
            'TOTALCLEARED': [150, 200]})
        self.trading_load['SETTLEMENTDATE'] = pd.to_datetime(self.trading_load['SETTLEMENTDATE'])
        self.dispatch_price = pd.DataFrame({
            'REGIONID': ['NSW1', 'NSW1', 'VIC1', 'VIC1'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00',
                              '2015/01/01 00:00:00', '2015/01/01 00:05:00'],
            'RRP': [99, 110,  300, 500]})
        self.dispatch_price['SETTLEMENTDATE'] = pd.to_datetime(self.dispatch_price['SETTLEMENTDATE'])
        self.trading_price = pd.DataFrame({
            'REGIONID': ['NSW1', 'VIC1'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:00:00'],
            'RRP': [99,  300]})
        self.trading_price['SETTLEMENTDATE'] = pd.to_datetime(self.trading_price['SETTLEMENTDATE'])
        self.region_summary = pd.DataFrame({
            'REGIONID': ['NSW1', 'NSW1', 'VIC1', 'VIC1'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00',
                               '2015/01/01 00:00:00', '2015/01/01 00:05:00'],
            'TOTALDEMAND': [5000, 5010,  8000, 8700]})
        self.region_summary['SETTLEMENTDATE'] = pd.to_datetime(self.region_summary['SETTLEMENTDATE'])

        self.expected_combined_df = pd.DataFrame({
            'DUID': ['A', 'A', 'B', 'B'],
            'DUDETAIL_EFFECTIVEDATE': ['2014/01/01 00:00:00', '2014/01/01 00:00:00',
                                       '2014/01/01 00:00:00', '2014/01/01 00:00:00'],
            'REGIONID': ['NSW1', 'NSW1', 'VIC1', 'VIC1'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00',
                               '2015/01/01 00:00:00', '2015/01/01 00:05:00'],
            'MAXCAPACITY': [333, 333, 250, 250],
            'SCADAVALUE': [150, 150, 200, 220],
            'TRADING_TOTALCLEARED': [150, math.nan, 200, math.nan],
            'DISPATCH_RRP': [99, 110,  300, 500],
            'TRADING_RRP': [99, math.nan, 300, math.nan],
            'TOTALDEMAND': [5000, 5010, 8000, 8700]})
        ix = pd.DatetimeIndex(start=datetime.strptime('2015/01/01 00:00:00', '%Y/%m/%d %H:%M:%S'),
                              end=datetime.strptime('2015/01/01 00:10:00', '%Y/%m/%d %H:%M:%S') - timedelta(minutes=5),
                              freq='5T')
        self.timeseries_df = pd.DataFrame(index=ix)
        self.timeseries_df.reset_index(inplace=True)
        self.timeseries_df.columns = ['SETTLEMENTDATE']
        self.expected_combined_df['SETTLEMENTDATE'] = pd.to_datetime(self.expected_combined_df['SETTLEMENTDATE'])
        self.expected_combined_df = self.expected_combined_df.sort_values('SETTLEMENTDATE')

    def test_merge_tables(self):
        merged_table = custom_tables.merge_tables_for_plant_stats(self.timeseries_df, self.gen_info, self.gen_info2,
                                                                  self.scada, self.trading_load,  self.dispatch_price,
                                                                  self.trading_price,  self.region_summary)
        np.array_equal(merged_table, self.expected_combined_df)


class TestSelectHighestVersionNumber(unittest.TestCase):
    def setUp(self):
        self.gen_info = pd.DataFrame({
            'EFFECTIVEDATE': ['2014/01/01 00:00:00', '2017/01/01 00:00:00', '2014/01/01 00:00:00',
                              '2014/01/01 00:00:00'],
            'VERSIONNO': ['1', '1', '1', '2'],
            'DUID': ['A', 'A', 'B', 'B'],
            'MAXCAPACITY': [333, 400, 250, 800]})
        self.gen_info['EFFECTIVEDATE'] = pd.to_datetime(self.gen_info['EFFECTIVEDATE'])
        self.expected_result = pd.DataFrame({
            'EFFECTIVEDATE': ['2014/01/01 00:00:00', '2017/01/01 00:00:00', '2014/01/01 00:00:00'],
            'VERSIONNO': ['1', '1', '2'],
            'DUID': ['A', 'A', 'B'],
            'MAXCAPACITY': [333, 400, 800]})
        self.expected_result['EFFECTIVEDATE'] = pd.to_datetime(self.expected_result['EFFECTIVEDATE'])

    def test_select_highest_version_no(self):
        result = custom_tables.select_highest_version_number(self.gen_info, defaults.table_primary_keys['DUDETAIL'])
        np.array_equal(result, self.expected_result)


class TestSelectInterventionIfPresent(unittest.TestCase):
    def setUp(self):
        self.dispatch_price = pd.DataFrame({
            'REGIONID': ['NSW1', 'NSW1', 'NSW1', 'VIC1', 'VIC1'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00',  '2015/01/01 00:05:00',
                               '2015/01/01 00:00:00', '2015/01/01 00:05:00'],
            'RRP': [99, 110, 112, 300, 500],
            'INTERVENTION': ['0', '0', '1', '0', '1']})
        self.dispatch_price['SETTLEMENTDATE'] = pd.to_datetime(self.dispatch_price['SETTLEMENTDATE'])
        self.expected_result = pd.DataFrame({
            'REGIONID': ['NSW1', 'NSW1', 'VIC1', 'VIC1'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00',
                               '2015/01/01 00:00:00', '2015/01/01 00:05:00'],
            'RRP': [99, 112,  300, 500],
            'INTERVENTION': ['0', '1', '0', '1']})
        self.expected_result['SETTLEMENTDATE'] = pd.to_datetime(self.expected_result['SETTLEMENTDATE'])

    def test_select_highest_version_no(self):
        result = custom_tables.select_intervention_if_present(self.dispatch_price,
                                                              defaults.table_primary_keys['DISPATCHPRICE'])
        np.array_equal(result, self.expected_result)


class TestPlantStats(unittest.TestCase):
    def setUp(self):
        pass

    def test_plant_stats(self):
        if os.path.isfile('C:/Users/user/Documents/plant_stats.csv'):
            t0 = time.time()
            plant_types = data_fetch_methods.static_table_xl('', '', 'Generators and Scheduled Loads', 'E:/raw_aemo_data',
                                                             select_columns=['DUID', 'Fuel Source - Primary',
                                                                             'Region', 'Participant'])
            plant_stats = custom_tables.plant_stats('2017/07/01 00:05:00', '2018/07/01 00:05:00', '', 'E:/raw_aemo_data')
            plant_stats = pd.merge(plant_stats, plant_types, 'left', 'DUID')
            plant_stats['TRADING_COST'] = plant_stats['Volume'] * plant_stats['TRADING_VWAP']
            plant_stats['DISPATCH_COST'] = plant_stats['Volume'] * plant_stats['DISPATCH_VWAP']
            plant_stats.to_csv('C:/Users/user/Documents/plant_stats_tp.csv')
            print(time.time() - t0)


class TestPlantsAgainstExcelNumbers(unittest.TestCase):
    def setUp(self):
        pass

    def test_nyngan1(self):
        if os.path.isfile('E:/plants_stats_test_data/NYNGAN1/NYNGAN1_test.xlsx'):
            xls = pd.ExcelFile('E:/plants_stats_test_data/NYNGAN1/NYNGAN1_test.xlsx')
            table = pd.read_excel(xls, 'Plant_stats', dtype=str)
            results = custom_tables.plant_stats('2017/01/01 00:00:00', '2017/02/01 00:00:00', '', 'E:/raw_aemo_data',
                                                filter_cols=['DUID'], filter_values=[('NYNGAN1',)])
            for col in [col for col in table.columns if col not in ['Month', 'DUID']]:
                table[col] = table[col].astype(float)
            results.reset_index(drop=True, inplace=True)
            pd.testing.assert_frame_equal(results, table)

    def test_eildon2(self):
        if os.path.isfile('E:/plants_stats_test_data/EILDON2/EILDON2_test.xlsx'):
            xls = pd.ExcelFile('E:/plants_stats_test_data/EILDON2/EILDON2_test.xlsx')
            table = pd.read_excel(xls, 'Plant_stats', dtype=str)
            results = custom_tables.plant_stats('2018/01/01 00:00:00', '2018/02/01 00:00:00', '', 'E:/raw_aemo_data',
                                                filter_cols=['DUID'], filter_values=[('EILDON2',)])
            for col in [col for col in table.columns if col not in ['Month', 'DUID']]:
                table[col] = table[col].astype(float)
            results.reset_index(drop=True, inplace=True)
            pd.testing.assert_frame_equal(results, table)


class TestCalcTradingLoad(unittest.TestCase):
    def setUp(self):
        self.scada = pd.DataFrame({
            'DUID': ['A', 'A', 'B', 'B'],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00',
                               '2015/01/01 00:00:00', '2015/01/01 00:05:00'],
            'SCADAVALUE': [150, 150,  200, 220]})
        self.scada['SETTLEMENTDATE'] = pd.to_datetime(self.scada['SETTLEMENTDATE'])
        self.result = pd.DataFrame({
            'DUID': ['A', 'A', 'B', 'B'],
            'TOTALCLEARED': [150, 150, 200, 220],
            'SETTLEMENTDATE': ['2015/01/01 00:00:00', '2015/01/01 00:05:00',
                               '2015/01/01 00:00:00', '2015/01/01 00:05:00']
            })
        self.result['SETTLEMENTDATE'] = pd.to_datetime(self.result['SETTLEMENTDATE'])

    def test_calc_trading_load_simple(self):
        trading_load = custom_tables.calc_trading_load(self.scada)
        pd.testing.assert_frame_equal(trading_load, self.result)


class TestCalcTradingLoad(unittest.TestCase):
    def setUp(self):
        pass

    def test_calc_trading_load_simple(self):
        pass
        #custom_tables.trading_and_dispatch_cost()




import pandas as pd
from datetime import timedelta, datetime
import math
import numpy as np
from nemosis import defaults, data_fetch_methods, filters


def fcas4s_scada_match(start_time, end_time, table_name, raw_data_location, select_columns=None, filter_cols=None,
                       filter_values=None):

    # Pull in the 4 second fcas data.
    table_name_fcas4s = 'FCAS_4_SECOND'
    fcas4s = data_fetch_methods.dynamic_data_compiler(start_time, end_time, table_name_fcas4s, raw_data_location)
    # Pull in the 4 second fcas variable types.
    table_name_variable_types = 'VARIABLES_FCAS_4_SECOND'
    fcas4s_variable_types = data_fetch_methods.static_table(start_time, end_time, table_name_variable_types,
                                                            raw_data_location)

    # Select the variable types that measure MW on an interconnector and Gen_MW from a dispatch unit.
    fcas4s_variable_types = fcas4s_variable_types[fcas4s_variable_types['VARIABLETYPE'].isin(['MW', 'Gen_MW'])]
    fcas4s = fcas4s[fcas4s['VARIABLENUMBER'].isin(fcas4s_variable_types['VARIABLENUMBER'])]

    # Select just the fcas 4 second data variable columns that we need.
    fcas4s = fcas4s.loc[:, ('TIMESTAMP', 'ELEMENTNUMBER', 'VALUE')]

    # Convert the fcas MW measured values to numeric type.
    fcas4s['VALUE'] = pd.to_numeric(fcas4s['VALUE'])

    # Rename the 4 second measurements to the timestamp of the start of the 5 min interval i.e round down to nearest
    # 5 min interval.
    fcas4s = fcas4s[(fcas4s['TIMESTAMP'].dt.minute.isin(list(range(0, 60, 5)))) &
                    (fcas4s['TIMESTAMP'].dt.second < 20)]
    fcas4s['TIMESTAMP'] = fcas4s['TIMESTAMP'].apply(lambda dt: datetime(dt.year, dt.month, dt.day, dt.hour, dt.minute))

    # Pull in the dispatch unit scada data.
    table_name_scada = 'DISPATCH_UNIT_SCADA'
    scada = data_fetch_methods.dynamic_data_compiler(start_time, end_time, table_name_scada, raw_data_location)
    scada['SETTLEMENTDATE'] = scada['SETTLEMENTDATE'] - timedelta(minutes=5)
    scada = scada.loc[:, ('SETTLEMENTDATE', 'DUID', 'SCADAVALUE')]
    scada.columns = ['SETTLEMENTDATE', 'MARKETNAME', 'SCADAVALUE']
    scada['SCADAVALUE'] = pd.to_numeric(scada['SCADAVALUE'])

    # Pull in the interconnector scada data and use the intervention records where the exist.
    table_name_inter_flow = 'DISPATCHINTERCONNECTORRES'
    inter_flows = data_fetch_methods.dynamic_data_compiler(start_time, end_time, table_name_inter_flow,
                                                           raw_data_location)
    inter_flows['METEREDMWFLOW'] = pd.to_numeric(inter_flows['METEREDMWFLOW'])
    inter_flows = inter_flows.sort_values('INTERVENTION')
    inter_flows = inter_flows.groupby(['SETTLEMENTDATE', 'INTERCONNECTORID'], as_index=False).last()
    inter_flows = inter_flows.loc[:, ('SETTLEMENTDATE', 'INTERCONNECTORID', 'METEREDMWFLOW')]
    inter_flows['SETTLEMENTDATE'] = inter_flows['SETTLEMENTDATE'] - timedelta(minutes=5)
    inter_flows.columns = ['SETTLEMENTDATE', 'MARKETNAME', 'SCADAVALUE']

    # Combine scada data from interconnectors and dispatch units.
    scada_elements = pd.concat([scada, inter_flows], sort=False)

    # Merge the fcas and scada data based on time stamp, these leads every scada element to be joined to every fcas
    # element that then allows them to be comapred.
    profile_comp = pd.merge(fcas4s, scada_elements, 'inner', left_on='TIMESTAMP', right_on='SETTLEMENTDATE')

    # Calculate the error between each measurement.
    profile_comp['ERROR'] = profile_comp['VALUE'] - profile_comp['SCADAVALUE']
    profile_comp['ERROR'] = profile_comp['ERROR'].abs()

    # Choose the fcas values that best matches the scada value during the 5 min interval.
    profile_comp = profile_comp.sort_values('ERROR')
    error_comp = profile_comp.groupby(['MARKETNAME', 'ELEMENTNUMBER', 'TIMESTAMP'], as_index=False).first()

    # Aggregate the error to comapre each scada and fcas element potential match.
    error_comp = error_comp.groupby(['MARKETNAME', 'ELEMENTNUMBER'], as_index=False).sum()

    # Sort the comparisons based on aggregate error.
    error_comp = error_comp.sort_values('ERROR')

    # Drop duplicates of element numbers and scada element names, keeping the record for each with the least error.
    best_matches_scada = error_comp[error_comp['SCADAVALUE'].abs() > 0]  # Don't include units 0 values for scada
    best_matches_scada = best_matches_scada.drop_duplicates('ELEMENTNUMBER', keep='first')
    best_matches_scada = best_matches_scada.drop_duplicates('MARKETNAME', keep='first')

    # Remove fcas elements where a match only occurred because both fcas and scada showed no dispatch.
    best_matches_scada['ELEMENTNUMBER'] = pd.to_numeric(best_matches_scada['ELEMENTNUMBER'])
    best_matches_scada = best_matches_scada.sort_values('ELEMENTNUMBER')
    best_matches_scada['ELEMENTNUMBER'] = best_matches_scada['ELEMENTNUMBER'].astype(str)

    # Give error as a percentage.
    best_matches_scada['ERROR'] = best_matches_scada['ERROR'] / best_matches_scada['SCADAVALUE']

    # drop matches with error greater than 100 %
    best_matches_scada = best_matches_scada[(best_matches_scada['ERROR'] < 1) & (best_matches_scada['ERROR'] > -1)]

    best_matches_scada = best_matches_scada.loc[:, ('ELEMENTNUMBER', 'MARKETNAME', 'ERROR')]

    if select_columns is not None:
        best_matches_scada = best_matches_scada.loc[:, select_columns]

    if filter_cols is not None:
        best_matches_scada = filters.filter_on_column_value(best_matches_scada, filter_cols, filter_values)

    return best_matches_scada


def capacity_factor(capacity_and_scada_grouped):
    scada_data = np.where(capacity_and_scada_grouped['SCADAVALUE'].isnull(), 0.0,
                          capacity_and_scada_grouped['SCADAVALUE'])
    cf = scada_data / capacity_and_scada_grouped['MAXCAPACITY']
    cf = cf.mean()
    return cf


def volume(capacity_and_scada_grouped):
    # Assumes 5 min Scada data
    return capacity_and_scada_grouped['SCADAVALUE'].sum()/12


def volume_weighted_average_price(output, prices):
    v_by_p = output * prices
    total_cash = v_by_p.sum()
    total_volume = output.sum()
    vwap = total_cash / total_volume
    return vwap


def volume_weighted_average_trading_price(capacity_and_scada_grouped):
    return volume_weighted_average_price(capacity_and_scada_grouped['TRADING_TOTALCLEARED'],
                                         capacity_and_scada_grouped['TRADING_RRP'])


def volume_weighted_average_spot_price(capacity_and_scada_grouped):
    return volume_weighted_average_price(capacity_and_scada_grouped['SCADAVALUE'],
                                         capacity_and_scada_grouped['DISPATCH_RRP'])


def alt_volume_weighted_average_trading_price(capacity_and_scada_grouped):
    return volume_weighted_average_price(capacity_and_scada_grouped['TRADING_TOTALCLEARED'],
                                         capacity_and_scada_grouped['TRADING_VWAP'])


def performance_at_nodal_peak(capacity_and_scada_grouped):
    index_max = capacity_and_scada_grouped['TOTALDEMAND'].idxmax()
    try:
        output_at_peak = capacity_and_scada_grouped['SCADAVALUE'][index_max]
    except:
        x=1
    if np.isnan(output_at_peak):
        output_at_peak = 0
    performance = output_at_peak / capacity_and_scada_grouped['MAXCAPACITY'][index_max]
    return performance


def capacity_factor_over_90th_percentile_of_nodal_demand(capacity_and_scada_grouped):
    data_entries = len(capacity_and_scada_grouped['TOTALDEMAND'])
    enteries_in_90th_percentile = math.ceil(data_entries/10)
    capacity_and_scada_grouped = capacity_and_scada_grouped.sort_values('TOTALDEMAND', ascending=False)
    capacity_and_scada_grouped = capacity_and_scada_grouped.reset_index(drop=True)
    capacity_and_scada_grouped = capacity_and_scada_grouped.iloc[:enteries_in_90th_percentile, :]
    cf = capacity_factor(capacity_and_scada_grouped)
    return cf


def stats_for_group(capacity_and_scada_grouped):
    cf = capacity_factor(capacity_and_scada_grouped)
    v = volume(capacity_and_scada_grouped)
    tvwap = volume_weighted_average_trading_price(capacity_and_scada_grouped)
    dvwap = volume_weighted_average_spot_price(capacity_and_scada_grouped)
    #alttvwap = alt_volume_weighted_average_trading_price(capacity_and_scada_grouped)
    peak = performance_at_nodal_peak(capacity_and_scada_grouped)
    peak_percentile = capacity_factor_over_90th_percentile_of_nodal_demand(capacity_and_scada_grouped)
    month = list(capacity_and_scada_grouped['MONTH'])[0]
    duid = list(capacity_and_scada_grouped['DUID'])[0]
    cf_df = pd.DataFrame({'Month': [month], 'DUID': [duid], 'CapacityFactor': [cf], 'Volume': [v],
                          'TRADING_VWAP': [tvwap], 'DISPATCH_VWAP': [dvwap], 'NodalPeakCapacityFactor': peak,
                          'Nodal90thPercentileCapacityFactor': [peak_percentile],
                          #'ALT_TRADING_VWAP': alttvwap
                          })
    return cf_df


def stats_by_month_and_plant(capacity_and_scada):
    capacity_and_scada['effective_set_date'] = capacity_and_scada['SETTLEMENTDATE'] #- timedelta(seconds=1)
    capacity_and_scada['MONTH'] = capacity_and_scada['effective_set_date'].dt.year.astype(str) + '-' + \
                                  capacity_and_scada['effective_set_date'].dt.month.astype(str).str.zfill(2)
    capacity_factors = capacity_and_scada.groupby(['MONTH', 'DUID'], as_index=False).apply(stats_for_group)
    return capacity_factors


def merge_tables_for_plant_stats(timeseries_df, gen_max_cap, gen_region, scada, trading_load, dispatch_price,
                                 trading_price, region_summary):
    gen_max_cap = gen_max_cap.sort_values('EFFECTIVEDATE')
    scada = scada.sort_values('SETTLEMENTDATE')
    merged_data = pd.merge_asof(scada, gen_max_cap, left_on='SETTLEMENTDATE', right_on='EFFECTIVEDATE', by='DUID')
    gen_region = gen_region.sort_values('START_DATE')
    merged_data = pd.merge_asof(merged_data, gen_region, left_on='SETTLEMENTDATE', right_on='START_DATE', by='DUID')
    merged_data = merged_data[~merged_data['REGIONID'].isnull()]
    merged_data = pd.merge(merged_data, trading_load, 'left', on=['SETTLEMENTDATE', 'DUID'])
    merged_data = pd.merge(merged_data, dispatch_price, 'left', on=['SETTLEMENTDATE', 'REGIONID'])
    merged_data = pd.merge(merged_data, trading_price, 'left', on=['SETTLEMENTDATE', 'REGIONID'])
    merged_data = pd.merge(merged_data, region_summary, 'left', on=['SETTLEMENTDATE', 'REGIONID'])
    merged_data = merged_data.loc[:, ('DUID', 'EFFECTIVEDATE', 'REGIONID', 'SETTLEMENTDATE', 'MAXCAPACITY',
                                      'SCADAVALUE', 'TOTALCLEARED', 'RRP_x', 'RRP_y', 'TOTALDEMAND')]
    merged_data.columns = ['DUID', 'DUDETAIL_EFFECTIVEDATE', 'REGIONID', 'SETTLEMENTDATE', 'MAXCAPACITY', 'SCADAVALUE',
                           'TRADING_TOTALCLEARED', 'DISPATCH_RRP', 'TRADING_RRP', 'TOTALDEMAND']
    return merged_data


def select_intervention_if_present(data, primary_key):
    data = data.sort_values(['INTERVENTION'])
    data = data.groupby([col for col in primary_key if col != 'INTERVENTION'], as_index=False).last()
    return data


def select_highest_version_number(data, primary_key):
    data['VERSIONNO'] = pd.to_numeric(data['VERSIONNO'])
    data = data.sort_values(['VERSIONNO'])
    data['VERSIONNO'] = data['VERSIONNO'].astype(int).astype(str)
    data = data.groupby([col for col in primary_key if col != 'INTERVENTION'], as_index=False).last()
    return data


def calc_trading_vwap_price(regional_demand, dispatch_price):
    demand_and_price = pd.merge(regional_demand, dispatch_price, 'inner', on=['SETTLEMENTDATE', 'REGIONID'])
    demand_and_price['COST'] = demand_and_price['RRP'] * demand_and_price['TOTALDEMAND']
    demand_and_price = demand_and_price.set_index('SETTLEMENTDATE')
    demand_and_price = demand_and_price.groupby('REGIONID').resample('30T', label='right', closed='right').aggregate(
        {'COST': 'sum', 'TOTALDEMAND': 'sum'})
    demand_and_price['TRADING_VWAP'] = demand_and_price['COST'] / demand_and_price['TOTALDEMAND']
    demand_and_price.reset_index(inplace=True)
    demand_and_price = demand_and_price.loc[:, ('SETTLEMENTDATE', 'REGIONID', 'TRADING_VWAP')]
    return demand_and_price


def calc_trading_load(scada):
    trading_load = scada.copy()
    trading_load = trading_load.set_index('SETTLEMENTDATE')
    trading_load['SCADAVALUE'] = pd.to_numeric(trading_load['SCADAVALUE'])
    trading_load = trading_load.groupby('DUID').resample('30T', label='right', closed='right').aggregate(
        {'SCADAVALUE': 'mean'})
    trading_load.reset_index(inplace=True)
    trading_load.columns = ['DUID', 'SETTLEMENTDATE', 'TOTALCLEARED']
    return trading_load


def calc_trading_price(dispatch_price):
    dispatch_price = dispatch_price.set_index('SETTLEMENTDATE')
    dispatch_price = dispatch_price.groupby('REGIONID').resample('30T', label='right', closed='right').aggregate({'RRP': 'mean'})
    dispatch_price.reset_index(inplace=True)
    return dispatch_price


def plant_stats(start_time, end_time, table_name, raw_data_location, select_columns=None, filter_cols=None,
                filter_values=None):

    ix = pd.DatetimeIndex(start=datetime.strptime(start_time, '%Y/%m/%d %H:%M:%S'),
                          end=datetime.strptime(end_time, '%Y/%m/%d %H:%M:%S') - timedelta(minutes=5),
                          freq='5T')
    timeseries_df = pd.DataFrame(index=ix)
    timeseries_df.reset_index(inplace=True)
    timeseries_df.columns = ['SETTLEMENTDATE']

    gen_max_cap = data_fetch_methods.dynamic_data_compiler(start_time, end_time, 'DUDETAIL', raw_data_location,
                                                           select_columns=['EFFECTIVEDATE', 'DUID', 'VERSIONNO',
                                                                           'MAXCAPACITY'], filter_cols=filter_cols,
                                                           filter_values=filter_values)
    gen_max_cap = select_highest_version_number(gen_max_cap, defaults.table_primary_keys['DUDETAIL'])
    gen_region = data_fetch_methods.dynamic_data_compiler(start_time, end_time, 'DUDETAILSUMMARY', raw_data_location,
                                                          select_columns=['START_DATE', 'END_DATE', 'DUID', 'REGIONID'],
                                                          filter_cols=filter_cols, filter_values=filter_values)
    scada = data_fetch_methods.dynamic_data_compiler(start_time, end_time, 'DISPATCH_UNIT_SCADA', raw_data_location,
                                                     select_columns=['SETTLEMENTDATE', 'DUID', 'SCADAVALUE'])
    dispatch_price = data_fetch_methods.dynamic_data_compiler(start_time, end_time, 'DISPATCHPRICE', raw_data_location,
                                                              select_columns=['SETTLEMENTDATE', 'REGIONID', 'RRP',
                                                                              'INTERVENTION'])
    dispatch_price = select_intervention_if_present(dispatch_price, defaults.table_primary_keys['DISPATCHPRICE'])
    trading_price = data_fetch_methods.dynamic_data_compiler(start_time, end_time, 'TRADINGPRICE', raw_data_location,
                                                             select_columns=['SETTLEMENTDATE', 'REGIONID', 'RRP'])
    dispatch_price['RRP'] = pd.to_numeric(dispatch_price['RRP'])
    trading_price['RRP'] = pd.to_numeric(trading_price['RRP'])
    #trading_price = calc_trading_price(dispatch_price)

    region_summary = data_fetch_methods.dynamic_data_compiler(start_time, end_time, 'DISPATCHREGIONSUM',
                                                              raw_data_location,
                                                              select_columns=['SETTLEMENTDATE', 'REGIONID',
                                                                              'TOTALDEMAND', 'INTERVENTION',
                                                                              'DISPATCHINTERVAL'])
    region_summary = select_intervention_if_present(region_summary, defaults.table_primary_keys['DISPATCHREGIONSUM'])

    scada_list = []
    for gen in scada.groupby(['DUID'], as_index=False):
        temp = pd.merge(timeseries_df, gen[1], 'left', on='SETTLEMENTDATE')
        temp['SCADAVALUE'] = np.where(temp['SCADAVALUE'].isnull(), 0.0, temp['SCADAVALUE'])
        temp['DUID'] = np.where(temp['DUID'].isnull(), gen[0], temp['DUID'])
        scada_list.append(temp)

    scada = pd.concat(scada_list)

    trading_load = calc_trading_load(scada)
    combined_data = merge_tables_for_plant_stats(timeseries_df, gen_max_cap, gen_region, scada, trading_load,
                                                 dispatch_price, trading_price, region_summary)
    combined_data['SCADAVALUE'] = pd.to_numeric(combined_data['SCADAVALUE'])
    combined_data['MAXCAPACITY'] = pd.to_numeric(combined_data['MAXCAPACITY'])
    combined_data['TRADING_TOTALCLEARED'] = pd.to_numeric(combined_data['TRADING_TOTALCLEARED'])

    combined_data['TOTALDEMAND'] = pd.to_numeric(combined_data['TOTALDEMAND'])

    stats = stats_by_month_and_plant(combined_data)
    return stats


def trading_and_dispatch_cost():
    gen_region = data_fetch_methods.dynamic_data_compiler('2017/01/01 00:05:00', '2018/01/01 00:05:00',
                                                          'DUDETAILSUMMARY', 'E:/raw_aemo_data',
                                                          select_columns=['START_DATE', 'END_DATE', 'DUID', 'REGIONID'])
    scada = data_fetch_methods.dynamic_data_compiler('2017/01/01 00:05:00', '2018/01/01 00:05:00',
                                                     'DISPATCH_UNIT_SCADA', 'E:/raw_aemo_data')

    ix = pd.DatetimeIndex(start=datetime.strptime('2017/01/01 00:00:00', '%Y/%m/%d %H:%M:%S'),
                          end=datetime.strptime('2018/01/01 00:00:00', '%Y/%m/%d %H:%M:%S'),
                          freq='5T')
    timeseries_df = pd.DataFrame(index=ix)
    timeseries_df.reset_index(inplace=True)
    timeseries_df.columns = ['SETTLEMENTDATE']

    scada_list = []
    for gen in scada.groupby(['DUID'], as_index=False):
        temp = pd.merge(timeseries_df, gen[1], 'left', on='SETTLEMENTDATE')
        temp['SCADAVALUE'] = np.where(temp['SCADAVALUE'].isnull(), 0.0, temp['SCADAVALUE'])
        temp['DUID'] = np.where(temp['DUID'].isnull(), gen[0], temp['DUID'])
        scada_list.append(temp)

    scada = pd.concat(scada_list)

    dispatch_price = data_fetch_methods.dynamic_data_compiler('2017/01/01 00:00:00', '2018/01/01 00:05:00',
                                                              'DISPATCHPRICE', 'E:/raw_aemo_data')
    dispatch_price = select_intervention_if_present(dispatch_price, defaults.table_primary_keys['DISPATCHPRICE'])
    gen_region = gen_region.sort_values('START_DATE')
    scada = scada.sort_values('SETTLEMENTDATE')
    scada_and_regions = pd.merge_asof(scada, gen_region, left_on='SETTLEMENTDATE', right_on='START_DATE', by='DUID')
    scada_and_regions = scada_and_regions[~scada_and_regions['REGIONID'].isnull()]
    scada_and_regions = pd.merge(scada_and_regions, dispatch_price, 'inner', left_on=['REGIONID', 'SETTLEMENTDATE'],
                                 right_on=['REGIONID', 'SETTLEMENTDATE'])

    scada_and_regions['SCADAVALUE'] = pd.to_numeric(scada_and_regions['SCADAVALUE'])
    scada_and_regions['RRP'] = pd.to_numeric(scada_and_regions['RRP'])

    scada_and_regions['DISPATCHCOST'] = (scada_and_regions['SCADAVALUE'] * scada_and_regions['RRP']) / 12

    scada_and_regions = scada_and_regions.set_index('SETTLEMENTDATE')
    scada_and_regions = scada_and_regions.groupby('DUID').resample('30T', label='right', closed='right').aggregate(
        {'SCADAVALUE': 'mean', 'RRP': 'mean', 'DISPATCHCOST': 'sum'})
    scada_and_regions.reset_index(inplace=True)

    scada_and_regions['TRADINGCOST'] = (scada_and_regions['SCADAVALUE'] * scada_and_regions['RRP']) / 2

    scada_and_regions['SCADAVALUE'] = scada_and_regions['SCADAVALUE'] / 2

    scada_and_regions = scada_and_regions.groupby('DUID').sum()

    scada_and_regions.to_csv('C:/Users/user/Documents/dispatch_trading_cost.csv')




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nemosis",
    version="1.2.1",
    author="Nicholas Gorman",
    author_email="n.gorman305@gmail.com",
    description="A tool for accessing AEMO data.",
    long_description="A tool for accessing AEMO data.",
    long_description_content_type="text/markdown",
    url="https://github.com/UNSW-CEEM/osdan",
    packages=setuptools.find_packages(),
    install_requires=['requests', 'joblib', 'pyarrow', 'feather-format', 'pandas'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ],
)


import tkinter as tk
import tkinter.ttk as ttk
from nemosis import defaults


class Query:

    def __init__(self, master, row_number, app, table_options=None):
        # Load in the starting features of a query row.
        self.master = master
        self.row_number = row_number

        # Make an label and entry box for the user to name the query result.
        self.query_label = ttk.Label(self.master, text='  Query name:')
        self.name = ttk.Entry(self.master)
        self.name.config(width=26)

        # Make labels and entry boxes for the user to provide start and end time to filter the query based on.
        self.start_time_label = ttk.Label(self.master, text='Start time:\n(YYYY/MM/DD HH:MM:SS)')
        self.start_time = ttk.Entry(self.master)
        self.start_time.config(width=26)
        self.end_time_label = ttk.Label(self.master, text='End time:\n(YYYY/MM/DD HH:MM:SS)')
        self.end_time = ttk.Entry(self.master)
        self.end_time.config(width=26)

        # Create a label and a list of tables to choose from.
        self.tables_label = ttk.Label(self.master, text='Select table:')
        self.tables = tk.Listbox(self.master, exportselection=False, width=35)
        self.tables.bind('<<ListboxSelect>>', self.add_column_selection)

        self.table_options = table_options

        for item in table_options:
            self.tables.insert(tk.END, item)

        # Create a button to delete the row.
        self.delete = ttk.Button(self.master, text=u"\u274C", command=lambda: app.delete_row(self.row_number))

        # Create empty attributes to fill up later on.
        self.filter_list = {}
        self.filter_label = {}
        self.filter_entry = {}
        self.col_list = None
        self.cols_label = None

        # Position all the widgets in the row.
        self.position()

    def position(self):
        first_sub_column = 0
        second_sub_column = 1
        pady = defaults.query_y_pad
        padx = defaults.standard_x_pad

        self.query_label.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number,
                              column=first_sub_column, pady=pady, padx=padx, sticky='sw')
        self.query_label.update()

        self.name.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                       + defaults.names_internal_row, column=first_sub_column, padx=padx, sticky='sw')
        self.name.update()

        self.start_time_label.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                       + defaults.start_time_label_internal_row, column=first_sub_column,
                                   padx=defaults.standard_x_pad, sticky='sw')
        self.start_time_label.update()

        self.start_time.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                             + defaults.start_time_internal_row, column=first_sub_column,
                             padx=padx, sticky='sw')
        self.start_time.update()

        self.end_time_label.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                 + defaults.end_time_label_internal_row, column=first_sub_column,
                                 padx=padx, sticky='sw')
        self.end_time_label.update()

        self.end_time.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                 + defaults.end_time_internal_row, column=first_sub_column,
                           padx=padx, sticky='sw')
        self.end_time.update()

        self.tables_label.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number,
                               column=second_sub_column, pady=defaults.query_y_pad, sticky='sw',
                               padx=padx)
        self.tables_label.update()

        self.tables.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                         + defaults.table_list_internal_row, column=second_sub_column,
                         rowspan=defaults.list_row_span, columnspan=defaults.list_column_span,
                         sticky='sw', padx=padx)
        self.tables.update()

        self.delete.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                         + defaults.delete_button_internal_row, column=defaults.last_column, sticky='nw')
        self.delete.update()

        if self.col_list is not None:
            self.position_column_list()
        if self.filter_list:
            self.position_filter_list()

    def add_column_selection(self, evt):
        # When a table is selected update the list of columns to be selected from.
        # Find the name of the table selected.
        table = self.table_options[self.tables.curselection()[0]]
        # Delete the previous list of columns.
        if self.col_list is not None:
            self.col_list.destroy()
            self.cols_label.destroy()

        # Create a new label and list box.
        self.cols_label = ttk.Label(self.master, text='Select columns:')
        self.col_list = tk.Listbox(self.master, selectmode=tk.MULTIPLE, exportselection=False, width=26)
        self.col_list.bind('<<ListboxSelect>>', self.add_filters)
        self.col_list.delete(0, tk.END)

        # Populate the list box with column names.
        for item in defaults.table_columns[table]:
            self.col_list.insert(tk.END, item)

        for col in defaults.table_primary_keys[table]:
            index = defaults.table_columns[table].index(col)
            self.col_list.selection_set(index)

        # Position the column list.
        self.position_column_list()

        # Delete any filters that existed for previous column selections.
        self.remove_filters()
        self.add_filters(None)

    def position_column_list(self):
        self.cols_label.grid(column=self.tables.grid_info()['column'] + defaults.list_column_span,
                             row=defaults.query_row_offset + self.row_number * defaults.row_height,
                             sticky='sw')
        self.cols_label.update()
        self.col_list.grid(column=self.tables.grid_info()['column'] + defaults.list_column_span,
                           row=defaults.query_row_offset + self.row_number * defaults.row_height
                           + defaults.table_list_internal_row,
                           rowspan=defaults.list_row_span, columnspan=defaults.list_column_span,
                           padx=defaults.standard_x_pad, sticky='sw')
        self.col_list.update()

    def add_filters(self, evt):
        # When a column is selected update the filter options.

        # Remove filters for columns that are no longer selected.
        self.remove_filters_unselected()

        # Find which columns are currently selected.
        select_cols = [self.col_list.get(0, tk.END)[index] for index in self.col_list.curselection()]

        # If a column is selected, and is filterable, but does not have a filter then add a filter for that column.
        for column in select_cols:
            if column in defaults.filterable_cols and column not in self.filter_label.keys():
                self.filter_label[column] = ttk.Label(self.master, text='Select {}s:'.format(str(column)))
                self.filter_entry[column] = ttk.Entry(self.master, width=25)
                self.filter_entry[column].bind('<Return>', self.add_to_list)
                self.filter_entry[column].name = column
                self.filter_list[column] = tk.Listbox(self.master, selectmode=tk.MULTIPLE, exportselection=False,
                                                      height=8, width=25)

        # Position all the filters so there are no gaps in between them.
        self.position_filter_list()

    def position_filter_list(self):
        last_filter = None
        # Position all the filter next to each other moving left across the screen.
        for column in self.filter_label.keys():
            if last_filter is None:
                # Place the first filter next to the column list.
                col = self.col_list.grid_info()['column'] + defaults.list_column_span
            else:
                # Place the next filter next to the last filter.
                col = self.filter_label[last_filter].grid_info()['column'] + defaults.list_column_span

            self.filter_label[column].grid(row=defaults.query_row_offset+ defaults.row_height * self.row_number,
                                           column=col, padx=defaults.standard_x_pad, sticky='sw')
            self.filter_label[column].update()
            self.filter_entry[column].grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                               + defaults.names_internal_row,
                                           column=col, padx=defaults.standard_x_pad)
            self.filter_entry[column].update()
            self.filter_list[column].grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                              + defaults.internal_filter_row, column=col,
                                          columnspan=defaults.list_column_span, rowspan=defaults.list_filter_row_span,
                                          padx=defaults.standard_x_pad)
            self.filter_list[column].update()
            last_filter = column

    def add_to_list(self, evt):
        # Add the item in the entry box of a filter to the list box below.
        self.filter_list[evt.widget.name].insert(tk.END, evt.widget.get())
        evt.widget.delete(0, 'end')

    def remove_filters_unselected(self):
        # Delete filter whoes columns are not selected.
        select_cols = [self.col_list.get(0, tk.END)[index] for index in self.col_list.curselection()]
        existing_filters = list(self.filter_label.keys())
        for column in existing_filters:
            if column not in select_cols:
                self.filter_label[column].destroy()
                del self.filter_label[column]
                self.filter_entry[column].destroy()
                del self.filter_entry[column]
                self.filter_list[column].destroy()
                del self.filter_list[column]

    def empty(self):
        # Delete all the widgets in the row.
        if self.filter_list:
            # Delete the filters.
            self.remove_filters()
        if self.col_list is not None:
            # Delete the column list.
            self.col_list.destroy()
            del self.col_list
        if self.cols_label is not None:
            self.cols_label.destroy()
            del self.cols_label
        # Delete the the row features that always exist.
        self.remove_initial_features()

    def remove_initial_features(self):
        # Remove the initial widegts.
        self.query_label.destroy()
        del self.query_label
        self.name.destroy()
        del self.name
        self.start_time_label.destroy()
        del self.start_time_label
        self.start_time.destroy()
        del self.start_time
        self.end_time_label.destroy()
        del self.end_time_label
        self.end_time.destroy()
        del self.end_time
        self.tables_label.destroy()
        del  self.tables_label
        self.tables.destroy()
        del self.tables
        self.delete.destroy()
        del self.delete

    def remove_filters(self):
        # Remove any filter widgets that exist.
        existing_filters = list(self.filter_label.keys())
        for column in existing_filters:
                self.filter_label[column].destroy()
                del self.filter_label[column]
                self.filter_entry[column].destroy()
                del self.filter_entry[column]
                self.filter_list[column].destroy()
                del self.filter_list[column]

    def state(self):
        # Return the current state of the row as a dictionary.
        state = {}
        state['type'] = 'query'
        state['name'] = self.name.get()
        state['start_time'] = self.start_time.get()
        state['end_time'] = self.end_time.get()
        if len(self.tables.curselection()) != 0:
            state['table'] = self.table_options[self.tables.curselection()[0]]
        if self.col_list is not None:
            state['columns'] = [self.col_list.get(0, tk.END)[index] for index in self.col_list.curselection()]
        state['filters_contents'] = {}
        state['filters_selection'] = {}
        for column, filter_list in self.filter_list.items():
            state['filters_contents'][column] = self.filter_list[column].get(0, tk.END)
            state['filters_selection'][column] = self.filter_list[column].curselection()
        return state

    def load_state(self, state):
        # Update the row to match the state provided.
        self.name.insert(0, state['name'])
        self.start_time.insert(0, state['start_time'])
        self.end_time.insert(0, state['end_time'])
        if len(state['table']) != 0:
            table_index = list(self.tables.get(0, "end")).index(state['table'])
            self.tables.selection_set(table_index)
            self.add_column_selection(None)
            for col in state['columns']:
                col_index = list(self.col_list.get(0, "end")).index(col)
                self.col_list.selection_set(col_index)
            self.add_filters(None)
        for column, filter_contents in state['filters_contents'].items():
            self.filter_list[column].insert(0, *filter_contents)
        for column, filter_selection in state['filters_selection'].items():
            for index in filter_selection:
                self.filter_list[column].selection_set(index)



class Merge:

    def __init__(self, master, row_number, app):
        # Create all the widgets of a merge row.
        self.master = master
        self.row_number = row_number
        # Create a label and entry box to name the result of the merge
        self.merge_label = ttk.Label(self.master, text='Merge name')
        self.name = ttk.Entry(self.master)
        self.name.config(width=26)
        # Create entry box to provide the name of the left result to merge.
        self.left_table_label = ttk.Label(self.master, text='Left table')
        self.left_table = ttk.Entry(self.master)
        self.left_table.config(width=26)
        # Create an entry box to provide the name of the right result to merge.
        self.right_table_label = ttk.Label(self.master, text='Right table')
        self.right_table = ttk.Entry(self.master)
        self.right_table.config(width=26)
        # Create a list to select the merge type from.
        self.join_types_label = ttk.Label(self.master, text='Select join type')
        self.join_types = tk.Listbox(self.master, exportselection=False, width=28)
        for item in defaults.join_type:
            self.join_types.insert(tk.END, item)
        # Create a button that deletes the row.
        self.delete = ttk.Button(self.master, text=u"\u274C", command=lambda: app.delete_row(self.row_number))
        # Create a entry box and list to provide the keys to the left result.
        self.left_keys_label = ttk.Label(self.master, text='Left keys')
        self.left_keys_entry = ttk.Entry(self.master)
        self.left_keys_entry.bind('<Return>', self.add_to_list_left)
        self.left_key_list = tk.Listbox(self.master, selectmode=tk.MULTIPLE, exportselection=False, height=8)
        # Create a entry box and list to provide the keys to the right result.
        self.right_keys_label = ttk.Label(self.master, text='Right keys')
        self.right_keys_entry = ttk.Entry(self.master)
        self.right_keys_entry.bind('<Return>', self.add_to_list_right)
        self.right_key_list = tk.Listbox(self.master, selectmode=tk.MULTIPLE, exportselection=False, height=8)
        # Position all the widgets.
        self.position()

    def position(self):
        self.merge_label.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number, column=0,
                              pady=defaults.query_y_pad, padx=defaults.standard_x_pad, sticky='ws')
        self.merge_label.update()
        self.name.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                           + defaults.names_internal_row, column=0, padx=defaults.standard_x_pad)
        self.name.update()
        self.left_table_label.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                       + defaults.start_time_label_internal_row, column=0, padx=defaults.standard_x_pad)
        self.left_table_label.update()
        self.left_table.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                 + defaults.start_time_internal_row, column=0, padx=defaults.standard_x_pad)
        self.left_table.update()
        self.right_table_label.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                        + defaults.end_time_label_internal_row, column=0, padx=defaults.standard_x_pad)
        self.right_table_label.update()
        self.right_table.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                                  + defaults.end_time_internal_row, column=0, padx=defaults.standard_x_pad)
        self.right_table.update()
        self.join_types_label.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number, column=1,
                               pady = defaults.query_y_pad, sticky = 'sw', padx=defaults.standard_x_pad)
        self.join_types_label.update()
        self.join_types.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                             + defaults.table_list_internal_row, column=1, rowspan=defaults.list_row_span,
                         columnspan=defaults.list_column_span, sticky='nw', padx=defaults.standard_x_pad)
        self.join_types.update()
        self.delete.grid(row=defaults.query_row_offset + defaults.row_height * self.row_number
                             + defaults.delete_button_internal_row, column=defaults.last_column, sticky='nw')
        self.delete.update()

        label_row = defaults.query_row_offset + defaults.row_height * self.row_number
        label_sticky = 'w'
        entry_row = defaults.query_row_offset + defaults.row_height * self.row_number + defaults.names_internal_row
        custom_list_row = defaults.query_row_offset + defaults.row_height * self.row_number + defaults.internal_filter_row

        left_keys_col = self.join_types.grid_info()['column'] + defaults.list_column_span
        self.left_keys_label.grid(row=label_row, column=left_keys_col, sticky=label_sticky, padx=defaults.standard_x_pad)
        self.left_keys_entry.grid(row=entry_row, column=left_keys_col, rowspan=defaults.list_row_span,
                                  columnspan=defaults.list_column_span, sticky='nw', padx=defaults.standard_x_pad)
        self.left_key_list.grid(row=custom_list_row, column=left_keys_col, rowspan=defaults.list_row_span,
                                columnspan=defaults.list_column_span,sticky='nw', padx=defaults.standard_x_pad)

        right_keys_col = self.left_key_list.grid_info()['column'] + defaults.list_column_span
        self.right_keys_label.grid(row=label_row, column=right_keys_col, sticky=label_sticky, padx=defaults.standard_x_pad)
        self.right_keys_entry.grid(row=entry_row, column=right_keys_col, rowspan=defaults.list_row_span, columnspan=defaults.list_column_span, sticky='nw',
                                  padx=defaults.standard_x_pad)
        self.right_key_list.grid(row=custom_list_row, column=right_keys_col, rowspan=defaults.list_row_span,columnspan=defaults.list_column_span,
                                sticky='nw', padx=defaults.standard_x_pad)

    def add_to_list_left(self, evt):
        # Add key from entry box to list.
        self.left_key_list.insert(tk.END, evt.widget.get())
        evt.widget.delete(0, 'end')

    def add_to_list_right(self, evt):
        # Add key from entry box to list.
        self.right_key_list.insert(tk.END, evt.widget.get())
        evt.widget.delete(0, 'end')

    def empty(self):
        # Delete the widgets of the merge row.
        self.merge_label.destroy()
        del self.merge_label
        self.name.destroy()
        del self.name
        self.left_table_label.destroy()
        del self.left_table_label
        self.left_table.destroy()
        del self.left_table
        self.right_table_label.destroy()
        del self.right_table_label
        self.right_table.destroy()
        del self.right_table
        self.join_types_label.destroy()
        del self.join_types_label
        self.join_types.destroy()
        del self.join_types
        self.left_keys_label.destroy()
        del self.left_keys_label
        self.left_keys_entry.destroy()
        del self.left_keys_entry
        self.left_key_list.destroy()
        del self.left_key_list
        self.right_keys_label.destroy()
        del self.right_keys_label
        self.right_keys_entry.destroy()
        del self.right_keys_entry
        self.right_key_list.destroy()
        del self.right_key_list
        self.delete.destroy()
        del self.delete

    def state(self):
        # Return the state of the row as a dictionary.
        state = {}
        state['type'] = 'merge'
        state['name'] = self.name.get()
        state['left_table'] = self.left_table.get()
        state['right_table'] = self.right_table.get()
        state['join_types'] = self.join_types.curselection()
        state['left_key_list'] = {}
        state['left_key_list']['contents'] = self.left_key_list.get(0, tk.END)
        state['left_key_list']['selection'] = self.left_key_list.curselection()
        state['right_key_list'] = {}
        state['right_key_list']['contents'] = self.right_key_list.get(0, tk.END)
        state['right_key_list']['selection'] = self.right_key_list.curselection()
        return state

    def load_state(self, state):
        # Update the row to match the state provided.
        self.name.insert(0, state['name'])
        self.left_table.insert(0, state['left_table'])
        self.right_table.insert(0, state['right_table'])
        if len(state['join_types']) != 0:
            self.join_types.selection_set(state['join_types'][0])

        self.left_key_list.insert(0, *state['left_key_list']['contents'])
        for index in state['left_key_list']['selection']:
            self.left_key_list.selection_set(index)

        self.right_key_list.insert(0, *state['right_key_list']['contents'])
        for index in state['right_key_list']['selection']:
            self.right_key_list.selection_set(index)